{% extends 'allianceauth/base-bs5.html' %}
{% load humanize %}
{% load i18n %}

{% block page_title %}{{ app_title }}{% endblock %}

{% block header_nav_brand %}{{ app_title }}{% endblock %}

{% block header_nav_user_character_control %}
    <div class="vr"></div>
{% endblock header_nav_user_character_control %}

{% block content %}{% endblock %}
